︠6c2ba507-791e-4ef5-af79-a6619deb4f60ai︠
%hide
%auto

from sage.ext.fast_eval import fast_float

class ff_vector:
    def __init__(self, vec):
        self.comp = []
        self.nice = []
        for expr in vec:
            self.nice.append(expr)
            self.comp.append(fast_float(expr))
    
    def __call__(self,*args):
        v = vector(RDF, [f(*args) for f in self.comp])
        return v
        
    def __repr__(self):
        return 'ff_vector: '+repr(self.nice)
︡a77ebe1c-1173-4338-b051-a987df40cc67︡︡
︠d5571a62-621f-4738-b184-10069805ed55ai︠
%hide
%auto

# 3-D motion and vector calculus
# Copyright 2009, Robert A. Beezer
# Creative Commons BY-SA 3.0 US
#
#
# 2009/02/15  Built on Sage 3.3.rc0
# 2009/02/17  Improvements from Jason Grout
#
# variable parameter is  t
# later at a particular value named t0
# 
# un-comment double hash (##) to get
# time-consuming torsion computation
#
var('t')

#
# parameter range
#
#start=float(-4*pi)
#stop=float(8*pi)
start=float(0)
stop=float(2*pi)
#
# position vector definition
# edit here for new example
# example is wide ellipse
# adjust figsize in final show() to get accurate aspect ratio
#
#a=float(1/(8*pi))
#c=(3/2)*a
#position=vector( (exp(a*t)*cos(t), exp(a*t)*sin(t), exp(c*t)) )
position=vector( (5/2*cos(t), 3/2*sin(t), 3/2*(t-pi)))
#position_print = vector( (exp(aa*t)*cos(t), exp(aa*t)*sin(t), exp(cc*t)) )
#
# graphic of the motion itself
#
path = parametric_plot3d( position(t).list(), (t, start, stop), color = "black" )
#
# derivatives of motion, lengths, unit vectors, etc
#
velocity = derivative( position ,t)
acceleration = derivative(velocity,t)
speed = velocity.norm()
speed_deriv = derivative(speed,t)
tangent = (1/speed)*velocity
dT = derivative(tangent,t)
normal = (1/dT.norm())*dT
binormal = tangent.cross_product(normal)
## dB = derivative(binormal,t)

velocity = ff_vector(velocity)
acceleration = ff_vector(acceleration)
speed = fast_float(speed)
speed_deriv = fast_float(speed_deriv)
tangent = ff_vector(tangent)
dT = ff_vector(dT)
normal = ff_vector(normal)
binormal = ff_vector(binormal)
augmenting_m = matrix(4,1,[0,0,0,1])
︡bd66c74d-c75d-4285-b0f3-af75ea4386d0︡{"stdout": "__main__:1: DeprecationWarning: Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)\n__main__:53: DeprecationWarning: Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)\n__main__:54: DeprecationWarning: Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)\n__main__:57: DeprecationWarning: Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)\n__main__:58: DeprecationWarning: Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)\n__main__:59: DeprecationWarning: Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)\n__main__:60: DeprecationWarning: Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)"}︡
︠8555a46b-dc54-439a-88e2-766baad54bea︠
#
# interact section
#   slider for parameter, 24 settings
#   checkboxes for various vector displays
#   computations at one value of parameter, t0
#

@interact
def _(
      pos_check = ("Position", True), 
      vel_check = ("Velocity", False),
      tan_check = ("Unit Tangent", False),
      nor_check = ("Unit Normal", False),
      bin_check = ("Unit Binormal", False),
      acc_check = ("Acceleration", False),
      tancomp_check = ("Tangential Component", False),
      norcomp_check = ("Normal Component", False),
      circle_check = ("Osculating Circle", False),
      t0 = slider(float(start), float(stop), float((stop-start)/24), float(start) , label = "Parameter"),
       ):
    #
    # location of interest
    #
    pos_tzero = position(t0)
    #
    # various scalar quantities at point
    #
    speed_component = speed(t0)
    tangent_component = speed_deriv(t0)
    normal_component = math.sqrt( acceleration(t0).norm()^2 - tangent_component^2 )
    curvature = normal_component/speed_component^2
    ## torsion = (1/speed_component)*(dB(t0)).dot_product(normal(t0))
 
    #
    # accumulate the graphic based on checkboxes
    #
    picture = path
    if pos_check:
        picture += arrow3d((0,0,0), pos_tzero, rgbcolor=(0,0,0))
    if vel_check:
        picture += arrow3d(pos_tzero, pos_tzero + velocity(t0), rgbcolor=(0,0.5,0))
    if tan_check:
        picture += arrow3d(pos_tzero, pos_tzero + tangent(t0), rgbcolor=(0,1,0) )
    if nor_check:
        picture += arrow3d(pos_tzero, pos_tzero + normal(t0), rgbcolor=(0.5,0,0))
    if bin_check:
        picture += arrow3d(pos_tzero, pos_tzero + binormal(t0), rgbcolor=(0,0,0.5))
    if acc_check:
        picture += arrow3d(pos_tzero, pos_tzero + acceleration(t0), rgbcolor=(1,0,1))
    if tancomp_check:
        picture += arrow3d(pos_tzero, pos_tzero + tangent_component*tangent(t0), rgbcolor=(1,0,1) )
    if norcomp_check:
        picture += arrow3d(pos_tzero, pos_tzero + normal_component*normal(t0), rgbcolor=(1,0,1))
    if circle_check:
        radius = 1.0/curvature
        m = identity_matrix(RDF,4)
        m[0:3,0] = -normal(t0)
        m[0:3,1] = tangent(t0)
        m[0:3,2] = binormal(t0)
        #m=matrix(zip(-normal(t0), tangent(t0), binormal(t0), pos_tzero+radius*normal(t0))).stack(matrix([0,0,0,1]))
        tx = sage.plot.plot3d.transform.Transformation(m=m)
        c = circle(center=(0,0), radius=radius, rgbcolor=(0.5,0,0)).plot3d()
        picture += c.transform(T=tx).translate(pos_tzero+radius*normal(t0))
    #
    # print textual info
    #
    print "Position vector: r(t)=", position(t)
    print "Speed is ", N(speed_component)
    print "Curvature is ", N(curvature)
    ## print "Torsion is ", N(torsion)

    #
    # show accumulated graphical info
    #
    show(picture, xmin=-3,xmax=3,ymin=-2,ymax=2,aspect_ratio=[1,1,1])
︡6b2c4a2d-bbb8-4f25-b3b8-1565b1f0b4ac︡︡
